var debug = (location.search.indexOf('debug') > 0) ? true : false;

(function(){
    //global
    var canvas,
        ctx,
        actx,
        timelast,
        //dt, //global
        width,
        height,

    //constants
        SPEED_DEFAULT,
        FIELD_WIDTH,
        //OUTER_FIELD_WIDTH, //global

    //settings
        controls,
        //landWidth, //global
        landValue,
        allowInput,
        menuShop,
        currentMenu,
        eyelids,
        quotes,
        tempIdlePosX,
        doonce,
        doonce2,

    //score
        money,
        stock,
        harvest,
        time,
        generations,

    //objects
        cam,
        farmer,
        family,
        house,
        barn,
        tools,
        soil,
        crops,
        oneups,

    //images
        imgBgFloor,
        imgBgFloorOuter,
        imgSoil,
        imgPlayerM,
        imgPlayerM0,
        imgPlayerM1,
        imgPlayerM2,
        imgItemHoe,
        imgItemBagWhea,
        imgItemBagCorn,
        imgItemBagCane,
        imgItemSickle,
        imgCrop,
        imgCropDead,
        imgDecoSign,
        imgDecoTomb,
        imgDecoSun,

    //sounds
        sfxCut,
        sfxCash,
        sfxPickup,
        sfxPlant,
        sfxStep,
        sfxSelect,
        sfxBlink;

    //game properties
    
    window.onload = function(){ 
        //setup objects once
        canvas = document.getElementById('game');
        ctx = canvas.getContext('2d');
        
        window.AudioContext = window.AudioContext || window.webkitAudioContext;
        actx = new AudioContext();

        onresize();
        doonce = false;

        /***** CONSTANTS *****/
        SPEED_DEFAULT = 0.3;
        FIELD_WIDTH = 1000;
        OUTER_FIELD_WIDTH = 300;

        /***** SETUP STAGE *****/

        //IMAGES
        //env
        imgBgFloor = loadImage('game/sprites/bg-floor.png');
        imgBgFloorOuter = loadImage('game/sprites/bg-dirt.png');
        imgSoil = loadImage('game/sprites/bg-soil.png');
        //players
        imgPlayerY = loadImage('game/sprites/player-y.png');
        imgPlayerO = loadImage('game/sprites/player-o.png');
        imgPlayerM = loadImage('game/sprites/player-m-sprite.png');
        imgPlayer0 = loadImage('game/sprites/player-m-0.png');
        imgPlayer1 = loadImage('game/sprites/player-m-1.png');
        imgPlayer2 = loadImage('game/sprites/player-m-2.png');
        //items
        imgItemHoe = loadImage('game/sprites/item-hoe.png');
        imgItemBagWhea = loadImage('game/sprites/seedbag-wheat.png');
        imgItemBagCorn = loadImage('game/sprites/seedbag-corn.png');
        imgItemBagCane = loadImage('game/sprites/seedbag-cane.png');
        imgItemSickle = loadImage('game/sprites/item-sickle.png');
        //crops
        imgCrop = [
            loadImage('game/sprites/crop-wheat.png'),
            loadImage('game/sprites/crop-corn.png'),
            loadImage('game/sprites/crop-cane.png')
        ];
        imgCropDead = loadImage('game/sprites/crop-dead.png');
        imgDecoSign = loadImage('game/sprites/sign-forsale.png');
        imgDecoTomb = loadImage('game/sprites/sign-tombstone.png');
        imgDecoSun = loadImage('game/sprites/sun.png');

        //SOUNDS
        sfxCut = new SoundObject('game/sounds/cut.wav', actx);
        sfxCash = new SoundObject('game/sounds/money.wav', actx);
        sfxPickup = new SoundObject('game/sounds/pickup.wav', actx);
        sfxPlant = new SoundObject('game/sounds/plant.wav', actx);
        sfxStep = new SoundObject('game/sounds/step.wav', actx);
        sfxSelect = new SoundObject('game/sounds/select.wav', actx);
        sfxBlink = new SoundObject('game/sounds/newday.wav', actx);

        //OBJECTS
        cam = {
            target: null,
            x: 0,
            targetPosX: 300
        };
        
        family = [new Farmer({
            ctx: ctx,
            image: imgPlayerM,
            mask: imgPlayer0,
            otherImages: [imgPlayerY, imgPlayerO],
            x: 380,
            y: 176,
            age: 5,
            sfx: [sfxStep]
        })];

        house = new Structure({
            ctx: ctx,
            image: loadImage('game/sprites/structure-house.png'),
            x: 200,
            width: 200,
            height: 200
        });

        barn = new Structure({
            ctx: ctx,
            image: loadImage('game/sprites/structure-barn.png'),
            x: 750,
            width: 200,
            height: 200
        });

        //tools
        tools = [
            {
                el: document.getElementById('tool-hoe'),
                infoEl: document.getElementById('tool-hoe-info'),
                tool: new Tool({
                    ctx: ctx,
                    image: imgItemHoe,
                    typeImage: [imgItemBagWhea, imgItemBagCorn, imgItemBagCane],
                    type: 0,
                    speed: 0.3,
                    action: function(){
                        for (var i=0; i<soil.length; i++) {
                            if (
                                farmer.x > (soil[i].x - soil[i].width) &&
                                farmer.x < (soil[i].x + soil[i].width)
                            ) {
                                
                                if (stock[this.type] > 0) {

                                    var exists = false;
                                    var seedling = false;
                                    for (var j=0; j<crops.length; j++) {
                                        if (soil[i].x === crops[j].x) {
                                            exists = true;
                                            if (crops[j].state === 0) {//only if it's still a seedling
                                                seedling = j;
                                            }
                                            break;
                                        }
                                    }

                                    if (exists) {
                                        if (typeof(seedling) == "number") {
                                            if (this.type === crops[seedling].type) {
                                                crops[seedling].increaseChance();
                                                updateStock(this.type, -1);
                                                updateStats(this.type, 1);

                                                oneUp('+xp', farmer.x + (Math.random() * 20 - 10), 150 + (Math.random() * 20 - 10), this.type);
                                                sfxPlant.play();
                                            }
                                        }    
                                    } else {
                                        crops.push(new Crop({
                                            ctx: ctx,
                                            type: this.type,
                                            x: soil[i].x,
                                            image: [imgCrop[this.type], imgCropDead]
                                        }));
                                        updateStock(this.type, -1);
                                        updateStats(this.type, 1);
                                        
                                        oneUp('+xp', farmer.x + (Math.random() * 20 - 10), 150 + (Math.random() * 20 - 10), this.type);
                                        sfxPlant.play();
                                    }
                                } else {
                                    updateStock(this.type, false);
                                    console.log("buy more seed: " + this.type);
                                }

                                break;
                            }
                        }
                    }
                })
            },
            {
                el: document.getElementById('tool-sickle'),
                infoEl: document.getElementById('tool-sickle-info'),
                tool: new Tool({
                    ctx: ctx,
                    image: imgItemSickle,
                    speed: 0.3,
                    action: function(){
                        for (var i=0; i<crops.length; i++) {
                            if (
                                farmer.x > (crops[i].x - crops[i].width/2) &&
                                farmer.x < (crops[i].x + crops[i].width/2)
                            ) {
                                
                                if (crops[i].state === 1) {
                                    updateStats(crops[i].type, 2);
                                    oneUp('+2xp', farmer.x + (Math.random() * 20 - 10), 150 + (Math.random() * 20 - 10), crops[i].type);
                                } else if (crops[i].state === -1) {
                                    updateStats(crops[i].type, 1);
                                    oneUp('+xp', farmer.x + (Math.random() * 20 - 10), 150 + (Math.random() * 20 - 10), crops[i].type);
                                }
                                crops[i].chop();

                                sfxCut.play();
                            }
                        }
                    }
                })
            }
        ];
        //farmer.item = tools[0].tool;

        soil = [];
        crops = [];
        oneups = [];

        quotes = [
            {
                text: "This Is Your World Too",
                time: 2000
            },
            {
                text: "By Andrew Kurniawan<br>@Pud9y",
                time: 2000
            },
            {
                text: "Use the arrow keys to move left and right",
                time: 6000
            },
            {
                text: "Hit spacebar to perform an action",
                time: 2000
            },
            {
                text: "\"And then one day, it wasn't just about me anymore...\"",
                time: 5000
            },
            {
                text: "\"Children learn so fast...\"",
                time: 2000
            },
            {
                text: "\"And when it's almost over, you just hope you've shown them enough.\"",
                time: 6000
            },
            {
                text: "\"...It will take everything, but I'll do it all for you...\"",
                time: 4000
            },
            {
                text: "\"After a hard days work, I need a good sleep.\"",
                time: 4000
            },
        ];
        

        /***** SETUP INTERFACE *****/
        window.onresize = onresize;

        //show first seed bag item
        document.getElementById('tool-hoe-info-type-0').style.display = "block";

        menuShop = [
            [
                {
                    el: document.getElementById('action-sleep'),
                    action: function() {
                        goToSleep();
                    }
                },
                {
                    el: document.getElementById('action-buy'),
                    action: function() {
                        menuShopOpen(1);
                    }
                },
                {
                    el: document.getElementById('action-sell'),
                    action: function() {
                        sellHarvest();
                    }
                },
                {
                    el: document.getElementById('action-buytools'),
                    action: function() {
                        menuShopOpen(2);
                    }
                }
            ],
            [
                {
                    el: document.getElementById('data-menu-shop-1-back'),
                    action: function() {
                        menuShopOpen(0);
                    }
                },
                {
                    el: document.getElementById('buy-seed-0'),
                    action: function() {
                        buySeed(0);
                    }
                },
                {
                    el: document.getElementById('buy-seed-1'),
                    action: function() {
                        buySeed(1);
                    }
                },
                {
                    el: document.getElementById('buy-seed-2'),
                    action: function() {
                        buySeed(2);
                    }
                }
            ],
            [
                {
                    el: document.getElementById('data-menu-shop-2-back'),
                    action: function() {
                        menuShopOpen(0);
                    }
                },
                {
                    el: document.getElementById('buy-item-0'),
                    action: function() {
                        //buyItem(0);
                    }
                },
                {
                    el: document.getElementById('buy-item-1'),
                    action: function() {
                        //buyItem(1);
                    }
                }
            ]
        ];
        //set house buttons actions
        for (var h=0; h<menuShop.length; h++) {
            for (var i=0; i<menuShop[h].length; i++) {
                menuShop[h][i].el.addEventListener('click', (function(h,i){
                    return function(e){
                        e.preventDefault();
                        menuShop[h][i].action();
                    }
                })(h,i));

                menuShop[h][i].el.addEventListener('touchstart', (function(h,i){
                    return function(e){
                        e.preventDefault();
                        menuShop[h][i].action();
                    }
                })(h,i));
            }
        }

        //set barn button actions
        var toolSelect = function(e){
            e.preventDefault();
            var n = parseInt(this.getAttribute('data-tool-id'));
            if (n || (n===0)) {
                selectTool(n);
            }
        };
        for (var i=0; i<tools.length; i++) {
            tools[i].el.addEventListener('click', toolSelect);
        }

        //set buyland buttons and select first item
        var btnbuyland = document.getElementById('buyland-purchase');
        btnbuyland.setAttribute('class','selected');
        btnbuyland.addEventListener('click', function(e){
            e.preventDefault();
            actionMenuBuyland();
        });

        var helptext = function(e){
            e.preventDefault();
            var text = 'Left / Right = Move\nUp/Down = Navigate Menus\nSpacebar  = Select/Action\n\nPlant seeds using the Hoe tool (found in the Barn). Move over the soil patches and press the action button to plant seed. Planting multiple seeds will improve chances of successful crop growth.\nGo to the house and sleep to let crops grow.\nHarvest using the Sickle.\nSell harvest to earn cash to expand your farm.';
            alert(text);
        }
        document.getElementById('btn-help').addEventListener('click', helptext);
        document.getElementById('btn-help').addEventListener('touchstart', helptext);



        /***** START NEW ROUND *****/
        eyelids = 0;
        landWidth = 950;
        newLand();

        landValue = 10000;

        farmer = family[0];
        cam.target = farmer;
        updateStats(0, Math.floor(Math.random()*50));
        updateStats(1, Math.floor(Math.random()*50));
        updateStats(2, Math.floor(Math.random()*50));

        money = 0;
        updateMoney(1000);
        stock = [0, 0, 0];
        updateStock(0,10);
        updateStock(1,0);
        updateStock(2,0);
        harvest = [0, 0, 0];
        updateHarvest(0,0);
        updateHarvest(1,0);
        updateHarvest(2,0);

        time = 0;
        updateTime(0);

        generations = []; //get generations.length to see how many



        /***** CONTROLS *****/
        allowInput = true;
        controls = {
            action: false,
            actionPressed: false,
            left: false,
            right: false,
            up: false,
            upPressed: false,
            down: false,
            downPressed: false
        };
        document.body.addEventListener('touchmove',function(e){e.preventDefault();}); //prevent touch app from dragging game screen

        document.body.addEventListener('keydown', function(e){
            var kc = e.keyCode;
            if (debug) { console.log(kc); }

            if (kc != 9 && kc != 91 && kc != 82 && kc != 13) { //not tab cmd or R or Enter
                e.preventDefault();    
            }
            
            if (kc == 37 || kc == 65) { //left & A
                controls.left = true;
            } else if (kc == 39 || kc == 68) { //right & D
                controls.right = true;
            }

            if (kc == 38 || kc == 87) {
                if (!controls.upPressed) {
                    controls.up = true;
                    controls.upPressed = true;
                }
            } else if (kc == 40 || kc == 83) {
                if (!controls.downPressed) {
                    controls.down = true;
                    controls.downPressed = true;
                }
            }

            if (kc == 32) { //space
                if (!controls.actionPressed) {
                    controls.actionPressed = true;
                    controls.action = true;
                }
            }
        });

        document.body.addEventListener('keyup', function(e){
            var kc = e.keyCode;

            if (kc != 9) {
                e.preventDefault();    
            }
            
            if (kc == 37 || kc == 65) { //left & A
                controls.left = false;
            } else if (kc == 39 || kc == 68) { //right & D
                controls.right = false;
            }

            if (kc == 38 || kc == 87) {
                controls.up = false;
                controls.upPressed = false;
            } else if (kc == 40 || kc == 83) {
                controls.down = false;
                controls.downPressed = false;
            }

            if (kc == 32) { //space
                controls.actionPressed = false;
                controls.action = false;
            }
        });

        document.body.addEventListener('touchstart', function(e) {
            touchstartX = e.touches[0].pageX;

            //also check that audio context is working first
            if (ios && !iOSAudioUnlocked) {
                // create empty buffer and play it
                var buffer = actx.createBuffer(1, 1, 22050);
                var source = actx.createBufferSource();
                source.buffer = buffer;
                source.connect(actx.destination);
                source.start(0);

                // by checking the play state after some time, we know if we're really unlocked
                setTimeout(function() {
                    if((source.playbackState === source.PLAYING_STATE || source.playbackState === source.FINISHED_STATE)) {
                        iOSAudioUnlocked = true;
                    }
                }, 0);
            }
        });
        document.body.addEventListener('touchmove', function(e){
            e.preventDefault();
            if (e.touches[0].pageX > (touchstartX + 20)) {
                controls.right = true;
            } else if (e.touches[0].pageX < (touchstartX - 20)) {
                controls.left = true;
            } else {
                controls.left = false;
                controls.right = false;
            }
        });
        document.body.addEventListener('touchend', function(e){
            if (e.changedTouches.length) {
                if (Math.abs(touchstartX - e.changedTouches[0].pageX) < 20) {
                    if(e.target == document.getElementById('interface')) {
                        controls.action = true;
                    }
                }
            }
            controls.left = false;
            controls.right = false;
        });
        document.querySelector('#tool-hoe-info>.tool-info:nth-child(2)').addEventListener('touchstart', function(){
            nextSeed();
        });

        


        /***** START LOOP *****/
        timelast = new Date().getTime();
        (function animloop() {
            requestAnimFrame(animloop);

            var ct = new Date().getTime();
            dt = (ct - timelast);
            timelast = ct;

            draw(dt);
        })();
    }


    /* Draw Loop */
    function draw(dt) {
        //clear frame
        ctx.clearRect(0, 0, width, height);

        if (debug) {console.log(totalAssets.loaded + '/' + totalAssets.loading);}

        if (totalAssets.loaded < totalAssets.loading) { //loading;
            ctx.fillStyle = "#000";
            ctx.fillRect(0,0,width,height);

            ctx.fillStyle = "#fff";
            ctx.fillRect(0, 0, (totalAssets.loaded/totalAssets.loading)*width,32);
        } else {

            if (!doonce) {
                doonce = true;

                quotePopup(quotes[0].text);
                setTimeout(function(){
                    quotePopup(quotes[1].text);
                }, 4000);

                tempIdlePosX = farmer.x;

                setTimeout(function(){
                    if (farmer.x === tempIdlePosX) { //if position is still the same, instruct movement
                        quotePopup(quotes[2].text, quotes[2].time);
                    }
                }, 12000);
            }


            if (ctx.imageSmoothingEnabled) {
                ctx.imageSmoothingEnabled = false;
            } else if (ctx.webkitImageSmoothingEnabled) {
                ctx.webkitImageSmoothingEnabled = false;
            } else if (ctx.webkitImageSmoothingEnabled) {
                ctx.mozImageSmoothingEnabled = false;
            }


            //CHECK INPUT
            if (allowInput) {
                eyelids += (160 - eyelids) * 0.1; //ease blackout

                if (controls.left || controls.right) { //if any horizontal controls
                    var toolspeed = (farmer.tool) ? farmer.tool.speed : SPEED_DEFAULT;
                    if (controls.left) {
                        farmer.vx += (-toolspeed - farmer.vx)*0.1;
                    }
                    if (controls.right) {
                        farmer.vx += (toolspeed - farmer.vx)*0.1;
                    }
                } else {
                    farmer.vx = farmer.vx * 0.6;
                }

                //in house
                var inShop = false;
                var winShop = document.getElementById('shop');
                if (
                    (farmer.x > (house.x - house.width/2)) &&
                    (farmer.x < (house.x + house.width/2)) 
                ){
                    if (winShop.getAttribute('data-show') != "true") { //show tool window
                        menuShopOpen(0);
                        winShop.setAttribute('data-show', 'true');
                    }

                    if (controls.up) {
                        controls.up = false;
                        prevMenuItem();
                    } else if (controls.down) {
                        controls.down = false;
                        nextMenuItem();
                    }

                    inShop = true;
                } else {
                    if (winShop.getAttribute('data-show') == "true") { //hide tool window
                        winShop.setAttribute('data-show', 'false');
                    }
                }

                //in barn
                var inTool = false;
                var winTool = document.getElementById('tool');
                if (
                    (farmer.x > (barn.x - barn.width/2)) &&
                    (farmer.x < (barn.x + barn.width/2)) 
                ){
                    if (winTool.getAttribute('data-show') != "true") { //show tool window
                        winTool.setAttribute('data-show', 'true');
                        //console.log(winTool.style);
                        if (!farmer.item) { //for the first time, select first tool
                            selectTool(0);
                        }
                    }

                    if (controls.up) {
                        controls.up = false;
                        prevTool();
                    } else if (controls.down) {
                        controls.down = false;
                        nextTool();
                    }

                    inTool = true;
                } else {
                    if (winTool.getAttribute('data-show') == "true") { //hide tool window
                        winTool.setAttribute('data-show', 'false');
                    }
                }

                var inBuyland = false;
                var winBuyland = document.getElementById('buyland');
                if (farmer.x >= (landWidth-farmer.width)) {
                    if (winBuyland.getAttribute('data-show') != "true") { //show tool window
                        winBuyland.setAttribute('data-show', 'true');
                    }

                    inBuyland = true;
                } else {
                    if (winBuyland.getAttribute('data-show') == "true") { //hide tool window
                        winBuyland.setAttribute('data-show', 'false');
                    }
                }


                if (
                    !inShop && 
                    !inTool &&
                    !inBuyland
                ) { //if not in either windows

                    if (controls.up) {
                        controls.up = false;

                        if ( //if it's a sowing tool
                            (farmer.item === tools[0].tool)
                        ) {
                            prevSeed();
                        }

                    }

                    if (controls.down) {
                        controls.down = false;

                        if ( //if it's a sowing tool
                            (farmer.item === tools[0].tool)
                        ) {
                            nextSeed();
                        }
                    }

                }

                if (controls.action) {
                    controls.action = false;

                    if (inShop) {
                        actionMenuItem();
                    } else if (inBuyland) {
                        actionMenuBuyland();
                    } else {
                        farmer.action();
                    }
                }

            } else {
                eyelids += (0 - eyelids) * 0.1; //ease blackout
            }




            //UPDATE
            for (var i=0; i<family.length; i++) {
                if (i > 0) {
                    family[i].vx += (farmer.vx*(Math.random() + 0.2) - family[i].vx)*0.1;
                    if (
                        ((family[i].vx > 0) && (family[i].x > farmer.x)) ||
                        ((family[i].vx < 0) && (family[i].x < farmer.x))
                    ) {
                        family[i].vx = 0;
                    }
                }
                family[i].update(dt);
            }

            //check for any collectable or deletable
            for (var i=crops.length-1; i>=0; i--) {
                if (crops[i].deletable) {
                    crops.splice(i,1); //delete it
                    continue;
                }
                if (
                    (crops[i].collectable) &&
                    (farmer.x > (crops[i].x - crops[i].width/2)) && 
                    (farmer.x < (crops[i].x + crops[i].width/2))
                ){  

                    //collect
                    var hType = crops[i].type;
                    var amt = Math.ceil(farmer.stats[hType]/20);
                    updateHarvest(hType, amt); //gain harvest based on skill level

                    oneUp('+' + amt, farmer.x + (Math.random() * 20 - 10), 150 + (Math.random() * 20 - 10), hType);
                    sfxPickup.play();
                    //console.log(harvest);

                    crops.splice(i,1); //delete it
                }
            }

            //tute checks
            if (!quotes[3].shown) {
                if (soil.length) {
                    if (farmer.x > soil[0].x) {
                        quotes[3].shown = true;
                        quotePopup(quotes[3].text);
                    }
                }
            }
            if (!quotes[8].shown) {
                if (stock[0] === 0) {
                    quotes[8].shown = true;
                    quotePopup(quotes[8].text, quotes[8].time);
                }
            }


            //RENDER
            ctx.fillStyle="#7ae3e8";
            ctx.fillRect(0,0, width, height);

            //center everything
            ctx.save();

                //camera
                cam.x += (cam.target.x - cam.x) * 0.1;
                if (cam.x < cam.targetPosX) {
                    cam.x = cam.targetPosX;
                } else if (cam.x > (landWidth + OUTER_FIELD_WIDTH - (width - cam.targetPosX))) {
                    cam.x = landWidth + OUTER_FIELD_WIDTH - (width - cam.targetPosX);
                }
                ctx.translate((-cam.x + cam.targetPosX), height/2 - 160);

                //bg
                ctx.fillStyle="#97f3f6";
                ctx.fillRect(0, 176, landWidth + OUTER_FIELD_WIDTH, 44);
                ctx.drawImage(imgBgFloorOuter, landWidth, 220, OUTER_FIELD_WIDTH, 100);
                
                //floor
                ctx.drawImage(imgBgFloor, 0, 220, landWidth, 100);

                ctx.drawImage(imgDecoSun, cam.x*0.99 + width - 500, 20, 40, 40);


                //objects
                

                //tombstone
                var startTombX = 600;
                for (var i=0; i<generations.length; i++) {
                    ctx.drawImage(generations[i], startTombX, 172, 56, 48);
                    startTombX -= 32;
                }
                
                //sale sign
                ctx.drawImage(imgDecoSign, landWidth + 32, 140, 80, 80);

                //house
                house.render();
                //barn
                barn.render();


                //soil
                for (var i=0; i<soil.length; i++) {
                    soil[i].render();
                }

                //player
                for (var i=0; i<family.length; i++) {
                    family[i].render();
                }

                //crops
                for (var i=0; i<crops.length; i++) {
                    crops[i].render();
                }


                //overlays
                for (var i=oneups.length-1; i>=0; i--) {
                    if (oneups[i].deletable) {
                        oneups.splice(i,1);
                    } else {
                        oneups[i].render();
                    }
                }



            ctx.restore();

            //blackout
            ctx.fillStyle = "#000";
            ctx.fillRect(0, 0, width, height/2 - eyelids);
            ctx.fillRect(0, height/2 + eyelids, width, height);
        }
    }
        

    /***** GAME FUNCTIONS *****/
    function selectTool(n) {
        for (var i=0; i<tools.length; i++) {
            tools[i].el.setAttribute('class', '');
            tools[i].infoEl.style.display = "none";
        }
        tools[n].el.setAttribute('class','selected');
        tools[n].infoEl.style.display = "block";

        farmer.item = tools[n].tool;

        sfxCash.play();
    }
    function currentTool() {
        var current = 0;
        for (var i=0; i<tools.length; i++) {
            if (tools[i].el.getAttribute('class') == "selected") {
                current = i;
                break;
            }
        }
        return current;
    }
    function prevTool() {
        var n = currentTool();
        n--;
        if (n<0) {
            n = tools.length-1;
        }
        selectTool(n);
    }
    function nextTool() {
        var n = currentTool();
        n++;
        if (n>tools.length-1) {
            n = 0;
        }
        selectTool(n);
    }

    function prevSeed() {
        farmer.item.type--;
        if (farmer.item.type < 0) {
            farmer.item.type = farmer.item.typeImage.length-1;
        }

        var allSeedTypes = document.querySelectorAll('.tool-hoe-info-type');
        for (var i=0; i<allSeedTypes.length; i++) {
            allSeedTypes[i].style.display = "none";
        }
        document.getElementById('tool-hoe-info-type-' + farmer.item.type).style.display = "block";
    }

    function nextSeed() {
        farmer.item.type++;
        if (farmer.item.type > farmer.item.typeImage.length-1) {
            farmer.item.type = 0;
        }

        var allSeedTypes = document.querySelectorAll('.tool-hoe-info-type');
        for (var i=0; i<allSeedTypes.length; i++) {
            allSeedTypes[i].style.display = "none";
        }
        document.getElementById('tool-hoe-info-type-' + farmer.item.type).style.display = "block";
    }

    function newLand() {
        var startPlant = landWidth + 100;
        for (var i=0; i<10; i++) {
            soil.push(newSoil(startPlant));
            startPlant += 80;
        }

        landWidth += FIELD_WIDTH;
    }
    function newSoil(x) {
        return {
            img: imgSoil,
            x: x,
            y: 216,
            width: 40,
            height: 16,
            render: function() {
                ctx.drawImage(this.img, this.x-20, this.y, this.width, this.height);
            }
        };
    }

    function menuShopOpen(n) {
        var allShops = document.querySelectorAll('[data-menu-shop-id]');
        for (var i=0; i<allShops.length; i++) {
            allShops[i].setAttribute('class','');
        }
        var open = document.querySelector('[data-menu-shop-id="' + n + '"]');

        if (open){
            open.setAttribute('class','current');
            currentMenu = n;
        } else {
            document.querySelector('[data-menu-shop-id="0"]');
            currentMenu = 0;
        }

        selectMenuItem(0);
    }
    function selectMenuItem(n) {
        var items = document.querySelectorAll('.current[data-menu-shop-id] a');
        if (items.length) {
            for (var i=0; i<items.length; i++) {
                items[i].setAttribute('class', '');
            }
            items[n].setAttribute('class','selected');
        }
    }
    function actionMenuItem() {
        var n = currentMenuItem();
        menuShop[currentMenu][n].action();

        sfxPlant.play();
    }
    function currentMenuItem() {
        var current = 0;
        var items = document.querySelectorAll('.current[data-menu-shop-id] a');
        for (var i=0; i<items.length; i++) {
            if (items[i].getAttribute('class') == "selected") {
                current = i;
                break;
            }
        }
        return current;
    }
    function prevMenuItem() {
        var items = document.querySelectorAll('.current[data-menu-shop-id] a');
        var n = currentMenuItem();
        n--;
        if (n<0) {
            n = items.length-1;
        }
        selectMenuItem(n);

        sfxSelect.play();
    }
    function nextMenuItem() {
        var items = document.querySelectorAll('.current[data-menu-shop-id] a');
        var n = currentMenuItem();
        n++;
        if (n>items.length-1) {
            n = 0;
        }
        selectMenuItem(n);

        sfxSelect.play();
    }

    function actionMenuBuyland() {
        if (money > landValue) {
            updateMoney(-landValue);
            newLand();

            landValue = landValue * 2;
            document.getElementById('buyland-value').innerHTML = landValue;
        } else {
            document.getElementById('money').setAttribute('class', 'error');
            setTimeout(function(){
                document.getElementById('money').setAttribute('class', '');
            }, 1000);
        }
        
    }



    function goToSleep() {
        allowInput = false;

        document.getElementById('shop').setAttribute('data-show','false');

        setTimeout(function(){
            //wake up

            allowInput = true;
            sfxBlink.play();

            updateTime(1);

            for (var i=0; i<crops.length; i++) {
                crops[i].grow();
            }

            for (var i=0; i<family.length; i++) {
                family[i].growUp();
            }

            //born spawn kid
            if (farmer.age == 10) {
                var rand = Math.floor(Math.random() * 3);
                var hat = imgPlayer0;
                if (rand == 2) {
                    hat = imgPlayer2;
                } else if (rand == 1) {
                    hat = imgPlayer1;
                }

                family.push(new Farmer({
                    ctx: ctx,
                    image: imgPlayerM,
                    mask: hat,
                    otherImages: [imgPlayerY, imgPlayerO],
                    x: 340,
                    y: 176,
                    age: 1,
                    sfx: [sfxStep]
                }));

                document.getElementById('stats-child').setAttribute('class', '');

                if (!quotes[4].shown) {
                    quotes[4].shown = true;
                    quotePopup(quotes[4].text, quotes[4].time);
                }
            }
            //die
            if (farmer.age >= 15) {
                //delete
                family.splice(0,1);

                generations.push(imgDecoTomb);

                //set new farmer target
                farmer = family[0];
                cam.target = farmer;

                //set stats to new farmer
                updateStats(0, 0, farmer.stats[0]);
                updateStats(1, 0, farmer.stats[1]);
                updateStats(2, 0, farmer.stats[2]);

                document.getElementById('child-0').innerHTML = 0;
                document.getElementById('child-1').innerHTML = 0;
                document.getElementById('child-2').innerHTML = 0;
                document.getElementById('stats-child').setAttribute('class', 'disabled');

                if (!quotes[7].shown) {
                    quotes[7].shown = true;
                    quotePopup(quotes[7].text, quotes[7].time);
                }
            }

            farmer.x = 380;
            farmer.vx = 0.1;

            //quote
            if (farmer.age >= 14) {
                if (!quotes[6].shown) {
                    quotes[6].shown = true;
                    quotePopup(quotes[6].text, quotes[6].time);
                }
            }


            //lose condition
            if (
                !money && 
                !stock[0] && 
                !stock[1] && 
                !stock[2] &&
                !harvest[0] &&
                !harvest[1] &&
                !harvest[2] &&
                !crops.length
            ) {
                alert('You\'re broke, and have failed your ' + generations.length + ' ancestors...');
                window.location.reload();
            }

            //win condition
            if (
                (landWidth > FIELD_WIDTH*10)
            ) {
                var string = 'Your ';
                if (generations.length > 2) {
                    for (var i=0; i<generations.length-2; i++) {
                        string += 'great ';
                    };
                    string += 'Grandparents would';
                } else if (generations.length > 0) {
                    string += 'Parents would';
                } else {
                    string = 'You should';
                }
                string += ' be proud for continuing this world for them.<br>YOU WIN!';
                
                quotePopup(string, 10000);
            }

        }, 2000);
    }
    function sellHarvest() {
        updateMoney(harvest[0] * 100);
        updateMoney(harvest[1] * 300);
        updateMoney(harvest[2] * 800);

        updateHarvest(0, 0, 0);
        updateHarvest(1, 0, 0);
        updateHarvest(2, 0, 0);

        //console.log('Total Cash: $' + money);
    }
    function buySeed(t) {
        var cost = 1000;
        if (t === 1) {
            cost = 2000;
        } else if (t === 2) {
            cost = 6000;
        }

        if (money >= cost) { //if enough for 10x, buy 10x
            updateMoney(-cost);
            updateStock(t,10);
        } else if (money >= (cost/10)) { //if not enough for 10x, buy 1x
            updateMoney(-cost/10);
            updateStock(t,1);
        } else {
            console.log('Insufficient funds. Try selling something.');
        }
    }

    //UI and score functions
    function updateMoney(s,r) {
        if (r || (r===0)) {
            money = r;
        } else if (s) {
            money += s;
        }
        document.getElementById('money').innerHTML = money;
        return money;
    }
    function updateStats(i,s,r) {
        if (i || (i===0)) {
            if (r || (r===0)) {
                farmer.stats[i] = r;

                for (var j=1;j<family.length;j++) {
                    family[j].stats[i] = r;
                }
            } else if (s) {
                farmer.stats[i] += s;

                for (var j=1;j<family.length;j++) {
                    family[j].stats[i] += s*2;

                    oneUp('+' + (s*2) + 'xp', family[j].x + (Math.random() * 20 - 10), 150 + (Math.random() * 20 - 10), i);

                    if (!quotes[5].shown) {
                        quotes[5].shown = true;
                        quotePopup(quotes[5].text, quotes[5].time);
                    }
                }
            }

            if (farmer.stats[i] > 100) {
                farmer.stats[i] = 100;
            }
            for (var j=1;j<family.length;j++) {
                if (family[j].stats[i] > 100) {
                    family[j].stats[i] = 100;
                }                
            }

            document.getElementById('skill-' + i).innerHTML = farmer.stats[i];
            document.getElementById('skill-bar-' + i).style.width = farmer.stats[i] + "px";

            if (family.length > 1) {
                document.getElementById('child-' + i).innerHTML = family[1].stats[i];
            }
            return farmer.stats[i];
        } else {
            return farmers.stats;
        }
    }
    function updateStock(i,s,r) {
        if (i || (i===0)) {
            var el = document.getElementById('stock-' + i);

            if (r || (r===0)) {
                stock[i] = r;
            } else if (s) {
                stock[i] += s;
            } else if (s === false) { //display error
                el.setAttribute('class', 'error');
                setTimeout(function(){
                    el.setAttribute('class', '');
                }, 1000);
            }

            el.innerHTML = stock[i];
            return stock[i];
        } else {
            return stock;
        }
    }
    function updateHarvest(i,s,r) {
        if (i || (i===0)) {
            if (r || (r===0)) {
                harvest[i] = r;
            } else if (s) {
                harvest[i] += s;
            }

            document.getElementById('harvest-' + i).innerHTML = harvest[i];
            return harvest[i];
        } else {
            return harvest;
        }
    }
    function updateTime(y) {
        if (y) {
            time += y;

            document.getElementById('time').innerHTML = time;
        }
        return time;
    }

    function oneUp(s, x, y, type) {
        var color = "#fff";
        if (type === 2) {
            color = "#5ca14d";
        } else if (type === 1) {
            color = "#fffe42";
        } else if (type === 0) {
            color = "#cda267";
        }

        var oup = {
            x: x || 0,
            y: y,
            targetY: y - 40,
            text: s,
            color: color,
            opacity: 1,
            deletable: false,
            render: function() {
                if (!this.deletable) {
                    this.y += (this.targetY - this.y) * 0.2;
                    this.opacity += (0 - this.opacity) * 0.05;

                    ctx.save();
                        ctx.globalAlpha = this.opacity;
                        ctx.fillStyle = this.color;
                        ctx.font="20px PressStart2P";
                        ctx.textAlign = "center";
                        ctx.fillText(this.text, this.x, this.y);
                    ctx.restore();

                    if (this.opacity < 0.01) {
                        this.deletable = true;
                    }
                }
            }
        };
        oneups.push(oup);
    }
     quotePopup = function(s, t) {
        var time = t || 2000;
        var qoute = document.getElementById('quote');
        quote.innerHTML = s;
        quote.style.display = "inline-block";
        setTimeout(function(){
            quote.setAttribute('class', 'fadein');
        }, 100);

        setTimeout(function(){
            quote.setAttribute('class', '');

            setTimeout(function(){
                quote.style.display = "none";
            }, 1000);
        }, time);
    }


    /***** COMMMON FUNCTIONS *****/
    function startGame(){
        
    }

    function endGame() {
        
    }

    function onresize() {
        if (canvas) {
            
            
            //dynamic sizing has been commented out
            canvas.width = window.innerWidth;
            // canvas.height = window.innerHeight;

            //canvas.width = 960;
            canvas.height = 640;
            width = canvas.width;
            height = canvas.height;

            // //special body scale when dynamic sizing is off
            // var scale = window.innerHeight / canvas.height;
            // document.body.setAttribute('style', 'transform: scale(' + scale + ',' + scale + ');');
        }
    }

    // shim layer with setTimeout fallback
    // Paul Irish - http://paulirish.com/2011/requestanimationframe-for-smart-animating/
    window.requestAnimFrame = (function () {
        return window.requestAnimationFrame       ||
               window.webkitRequestAnimationFrame ||
               window.mozRequestAnimationFrame    ||
            function (callback) {
                window.setTimeout(callback, 1000 / fps);
            };
    })();

})();